/*********************************************************************
 * INCLUDES
 */
#include "bsp_adc.h"
#include "system.h"
#include "exception.h"
/*********************************************************************
 * DEFINITIONS
 */
 #define CHANNEL_MAX							8
 #define PIN_NOT_USED							0xFF
 #define CHANNEL_NOT_USED					0xFF
 #define WAIT_TIME_VALUE					10000								// ȴʱֵ
/*********************************************************************
 * STRUCTION
 */
typedef struct {
	uint32_t pin;
	uint32_t channel;
}adc_config_t;
/*********************************************************************
 * LOCAL VARIABLES
 */
static adc_config_t									m_adc_config[CHANNEL_MAX] 	= {{PIN_NOT_USED,CHANNEL_NOT_USED},{PIN_NOT_USED,CHANNEL_NOT_USED},{PIN_NOT_USED,CHANNEL_NOT_USED},{PIN_NOT_USED,CHANNEL_NOT_USED},{PIN_NOT_USED,CHANNEL_NOT_USED},{PIN_NOT_USED,CHANNEL_NOT_USED},{PIN_NOT_USED,CHANNEL_NOT_USED},{PIN_NOT_USED,CHANNEL_NOT_USED}};
static nrf_saadc_channel_config_t 	channelConfig[CHANNEL_MAX];
static nrf_saadc_value_t 						s_bufferPool[CHANNEL_MAX] 	= {0};
static volatile bool 								adc_SampleOk 								= true;		// adcɼɱ־
//SAADCݻ
//SAADCС
//ֻвû֮,ŻSAADC¼
static uint32_t 										sample_in_buffer 						= 0;
/*********************************************************************
 * LOCAL FUNCTIONS
 */
/**
 @brief ADCжϴص
 @param 
 @return 
*/
static void adcCallbackFunc(nrf_drv_saadc_evt_t const *pEvent)
{
	ret_code_t err_code;
	if(pEvent->type == NRF_DRV_SAADC_EVT_DONE)	// ɣɼʱ˳ΪͨС䡣
	{
		//úû棬Ϊһβ׼
		err_code=nrf_drv_saadc_buffer_convert(pEvent->data.done.p_buffer,sample_in_buffer);
		if(err_code == NRF_SUCCESS)
		{
			adc_SampleOk = true;
		}
	}
}

static void bsp_adc_init_process(void)
{
	if(Except_TxError(EXCEPT_ADC_INIT,"bsp_adc_init_error\r\n") == 0)
	{
		Process_Stop(bsp_adc_init_process);
	}
}

/**
	@brief	ʼADC
	@param	
	@return 
*/
static void ADC_Init(void)
{
	int 				ret = 0;
	ret_code_t 	errCode;
	
	nrf_drv_saadc_config_t p_config = NRFX_SAADC_DEFAULT_CONFIG;
	p_config.interrupt_priority = ADC_IRQ_PRIORITY;
	
	
	// ADCʼ
	errCode = nrf_drv_saadc_init(&p_config, adcCallbackFunc);//ȼΪ3ȶʱжҪߣȻصڶʱжϽ󴥷
	if(errCode != NRF_SUCCESS)ret = -1;
	
	// ADCͨ
	for(int i=0;i<CHANNEL_MAX;i++)
	{
		if(m_adc_config[i].pin != PIN_NOT_USED && m_adc_config[i].channel != CHANNEL_NOT_USED)
		{
			// ADCͨʼ
			errCode = nrf_drv_saadc_channel_init(m_adc_config[i].channel, &channelConfig[m_adc_config[i].channel]);
			if(errCode != NRF_SUCCESS)ret = -1;
		}
	}
	
	if(sample_in_buffer > 0)
	{
		// 
		errCode = nrf_drv_saadc_buffer_convert(s_bufferPool, sample_in_buffer);
		if(errCode != NRF_SUCCESS)ret = -1;
	}
	
	
	if(ret == -1)
	{
		Process_Start(0,"bsp_adc_init_process",bsp_adc_init_process);
	}
}
static void cb_adcWakeup(uint32_t t)
{
	ADC_Enable(); 
}

static void cb_adcSleep(uint32_t t)
{
	ADC_Disable();
}

/*********************************************************************
 * PUBLIC FUNCTIONS
 */
/**
	@brief	ʼADCźͨ
	@param	pin -[in]	Ҫʼadc
	@param	channel -[in]	Ҫʼadcͨ
	@return 
*/
uint32_t ADC_SetPinChannel(uint32_t pin, uint32_t channel,nrf_gpio_pin_pull_t pin_pull)
{
	ret_code_t 	errCode;
	
	//Ѵڵźͨ
	ADC_RemovePinChannel(pin, channel);
	
	for(int i=0;i<CHANNEL_MAX;i++)
	{
		if(m_adc_config[i].pin == PIN_NOT_USED && m_adc_config[i].channel == CHANNEL_NOT_USED && PIN_NOT_USED != pin)	
		{
			m_adc_config[i].pin = pin;
			m_adc_config[i].channel = channel;
			
			//ADCΪ
			nrf_gpio_cfg_input(m_adc_config[i].pin,pin_pull);
			// 
			channelConfig[m_adc_config[i].channel].resistor_p = NRF_SAADC_RESISTOR_DISABLED;
			channelConfig[m_adc_config[i].channel].resistor_n = NRF_SAADC_RESISTOR_DISABLED;
			channelConfig[m_adc_config[i].channel].gain       = NRF_SAADC_GAIN1_6;
			channelConfig[m_adc_config[i].channel].reference  = NRF_SAADC_REFERENCE_INTERNAL;
			channelConfig[m_adc_config[i].channel].acq_time   = NRF_SAADC_ACQTIME_10US;
			channelConfig[m_adc_config[i].channel].mode       = NRF_SAADC_MODE_SINGLE_ENDED;
			channelConfig[m_adc_config[i].channel].burst      = NRF_SAADC_BURST_DISABLED;
			channelConfig[m_adc_config[i].channel].pin_p      = (nrf_saadc_input_t)(m_adc_config[i].channel + 1);
			channelConfig[m_adc_config[i].channel].pin_n      = NRF_SAADC_INPUT_DISABLED;
			// ADCͨʼ
			errCode = nrf_drv_saadc_channel_init(m_adc_config[i].channel, &channelConfig[m_adc_config[i].channel]);
			if(errCode != NRF_SUCCESS)return ADC_ERR_INIT_CONGESTION;
			
			sample_in_buffer++;
			
			SEGGER_RTT_printf(0, "add channel (  %d  ):\n", channel);

			return ADC_OP_SUCCESS;
		}
	}
	return ADC_ERR_INIT_CONGESTION;
}
/**
	@brief	ƳѳʼADCźͨ
	@param	pin -[in]	ҪƳadc
	@param	channel -[in]	ҪƳadcͨ
	@return 
*/
uint32_t ADC_RemovePinChannel(uint32_t pin, uint32_t channel)
{
	for(int i=0;i<CHANNEL_MAX;i++)
	{
		if(m_adc_config[i].pin == pin && m_adc_config[i].channel == channel)
		{
			nrf_drv_saadc_channel_uninit(m_adc_config[i].channel);
			m_adc_config[i].pin = PIN_NOT_USED;
			m_adc_config[i].channel = CHANNEL_NOT_USED;
			if(sample_in_buffer != 0)sample_in_buffer--;
			
			SEGGER_RTT_printf(0, "del channel (  %d  ):\n", channel);
			return ADC_OP_SUCCESS;
		}
	}
	return ADC_ERR_REMOVE_PIN_CHANNEL;
}
/**
	@brief ADCʼ
	@param 
	@return 
*/
void ADC_Initialize(void)
{
	ADC_SetPinChannel(PIN_ADC_BAT_IN, PIN_ADC_BAT_CHANNEL,NRF_GPIO_PIN_NOPULL);
	ADC_SetPinChannel(PIN_ADC_CHARGMEASURE, PIN_ADC_CHARGMEASURE_CHANNEL,NRF_GPIO_PIN_NOPULL);
	ADC_Init();
	Wakeup_Regist(cb_adcWakeup);
	Sleep_Regist(cb_adcSleep);
}
/**
	@brief ADCȡ
	@param channel -[in] Ҫȡͨ
	@param p_adc_value -[out] ضȡֵͨ
	@return 
*/
uint32_t ADC_Read(uint32_t channel, int16_t *p_adc_value)
{
	uint32_t errCode;
	uint32_t wait_time_out = WAIT_TIME_VALUE;
	uint8_t  adc_sort[CHANNEL_MAX] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
	uint8_t  sort_index = 0;
	int  		 i;
	adc_SampleOk = false;
	
	//ûͨʧ
	if(sample_in_buffer == 0)return ADC_ERR_READ_NO_CHANNEL;
	
	errCode = nrf_drv_saadc_sample();
	if(errCode != NRF_SUCCESS)return ADC_ERR_READ_FAIL;
	
	//ȴɼ
	while(!adc_SampleOk)
	{
		if(wait_time_out--){
//			nrf_pwr_mgmt_run();
		}
		else{
			return ADC_ERR_READ_TIMEOUT;
		}
	}
	//Ԥ򣬲ɼʱ˳ΪͨС䡣
	for(i=0;i<CHANNEL_MAX;i++)
	{
		if(channelConfig[i].pin_p != NRF_SAADC_INPUT_DISABLED){
			adc_sort[sort_index] = i;
			sort_index++;
		}
	}
	//ȡ
	for(i=0;i<sample_in_buffer;i++)
	{
		if(adc_sort[i] == channel)
		{
			*p_adc_value = s_bufferPool[i];
		}
	}

	return ADC_OP_SUCCESS;
}
/**
 @brief ADCʼûΪDisableɶԳ
 @param 
 @return 
*/
void ADC_Enable(void)
{
	ADC_Init();
}
/**
 @brief ADC
 @param 
 @return 
*/
void ADC_Disable(void)
{
	
	nrf_drv_saadc_uninit();
}






